@@ -17,6 +17,7 @@
 #include <linux/console.h>
 #include <asm/types.h>
 #include "fbcon.h"
+
 #ifdef CONFIG_UNICON
 #include "unicon.c"
 extern struct display fb_display[MAX_NR_CONSOLES];
@@ -356,15 +357,22 @@ static void bit_putcs_wrap(struct vc_data *vc, struct fb_info *info,
 		return;
 	}
 
+	/*
+	 * 2009/01/23
+	 * modified by Toshi for gpm
+	 */
 	if(count == 1)
 	{
-		int extendedchar, index;
-		u_char * fontdata_save;
-		int c = *(int*)s;
+		u32 extendedchar, index, c;
+		u8* fontdata_save;
+		u16 chr[2] = {0, 0};
+
+		fontdata_save = vc->vc_font.data;
+		c = (s >= (u16*)vc->vc_origin) && (s < (u16*)vc->vc_scr_end);
+		c = c ? (scr_readw (hibyte_pos (vc, s)) << 16) | scr_readw (s) : (s[1] << 16) | s[0];
 	
 		extendedchar = c & (DB_VALIDATE<<16);
 		if (extendedchar) {
-			fontdata_save = vc->vc_font.data;
 			if ( c & (DB_RIGHT_MASK << 16)) {// right half
 				index = doublebyte_default->font_index((c>>16)&0xff, (c&0xff)) + 16;
 			} else {
@@ -373,15 +381,14 @@ static void bit_putcs_wrap(struct vc_data *vc, struct fb_info *info,
 
 			if (index >=0 && index < doublebyte_default->charcount) {
 				vc->vc_font.data = doublebyte_default->font_data +index;
-				c = (c & 0xff00);
-			}else c = c & 0xffff;
-
-			bit_putcs(vc, info, &c, 1, yy, xpos, fg, bg);
-			vc->vc_font.data = fontdata_save;
-		} else {
-			c = c & 0xffff;
-			bit_putcs(vc, info, &c, 1, yy, xpos ,fg ,bg);
+				c &= 0xff00;
+			}
 		}
+
+		chr[0] = (u16)c;
+		bit_putcs(vc, info, &chr[0], 1, yy, xpos, fg, bg);
+
+		vc->vc_font.data = fontdata_save;
 		return;
 	}
 
